function [ b, res, wssr, it ] = irwls(y,X,wfunc,param,start)
% irwls    iteratively re-weighted least squares
%          Can solve "robust" regression problems using
%          robust weights.
% Usage:
%        [b, res, wssr, it ] = irwls(y,X,wfunc,param,start)
%
% Arguments:
%        y	n-vector of data
%        X      nxp design matrix
%        wfunc	the name of the weight function subroutine, passed as a string
%               W = wfunc(res, scale, param) must take three arguments:
%			res 	vector of residuals
%			scale	resistant estimate of scale (MAD/.6745)
%			param	array of particular parameters passed by caller
%		wfunc must return a vector W of the same dimension as res
%        param	a vector of parameters to pass to wfunc.
%		DEFAULT = 1.
%	 start	a flag for what to use as a starting model to get
%		the estimate of scale that is passed to the weight
%		function. To get the estimate of scale, the algorithm
%		uses the median absolute deviation (MAD) of the residuals,
%		standardized to that of the normal distribution.
%		If start = 'ols', the median absolute deviation of
%		the residuals from the ordinary least-squares regression
%		function are used.
%		If start = 'median', the median absolute deviation
%		of the residuals from the median are used, completely
%		ignoring the possible variation of the "true" function.
%		If start = 'one-step', the first estimate of scale
%		is the same as for 'median', but the scale estimate is
%		updated based on the residuals from the first step
%		of the least squares fit using that scale (the scale estimate
%		is only updated once, after the first wls).
%		DEFAULT = 'ols'
%
% Results:
%        b	p-vector of fitted coefficients
%        res	n-vector of final residuals
%        wssr	final sum of squared weighted residuals
%	 it	number of iterations taken

% P.B. Stark 	stark@stat.berkeley.edu
% 9 July 1997

chgTol = 0.001;					% tolerance for convergence 
						% of coefficients
maxit = 500;					% maximum number of iterations

% check dimensions
[n,p] = size(X);
[ny, py] = size(y);

if (n ~= ny),
	error('number of rows of y ~= number of rows of X')
end

if (py ~= 1),
	error('y has more than one column')
end

if (nargin < 3),
	param = 1;
	start = 'ols';
elseif (nargin < 4),
	start = 'ols';
end

if (~(strcmp(start,'ols') | strcmp(start,'median') | ...
	strcmp(start,'one-step'))),
	error(['start = ', start, ' not supported'])
end

%
if (strcmp(start,'ols')),
	W = ones(n,1);				% initial weight matrix
	[b, res, wssr] = wls(y,X,W);		% initial regression
	scale = 1.483*median(abs(res-median(res)));
elseif (strcmp(start,'median')),
	res = y - median(y);
	scale = 1.4843*median(abs(res-median(res)));
	W = diag(sqrt(feval(wfunc,res,scale,param)));
	[b, res, wssr] = wls(y, X, W);
elseif (strcmp(start,'one-step')),
	res = y - median(y);
	scale = 1.4843*median(abs(res-median(res)));
	W = diag(sqrt(feval(wfunc,res,scale,param)));
	[b, res, wssr] = wls(y, X, W);
	scale = 1.4843*median(abs(res-median(res)));
end

% loop
relchg = 1; 					% relative change in coefficients
it = 0;
while (relchg > chgTol & it < maxit),    	% stop when the relative change in
						% regression coefficients between
						% iterations is small enough
	lastB = b;
	it = it + 1;
        W = diag(sqrt(feval(wfunc,res,scale,param)));
	[b, res, wssr] = wls(y, X, W);
	relchg = max( abs( b - lastB ) ./ abs(b));
end
if (it == maxit ),
	error(['maximum number of iterations (' num2str(maxit) ...
		 ') exceeded'])
end
return